package com.ruoyi.hezhi.service;

import com.github.pagehelper.PageInfo;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.hezhi.domain.TbCourse;
import com.ruoyi.hezhi.domain.TbSubjectWarehouseClass;
import com.ruoyi.hezhi.domain.dto.CourseDTO;

import java.util.List;
import java.util.Map;

/**
 * 课程Service接口
 *
 * @author ruoyi
 * @date 2024-09-14
 */
public interface ITbCourseService
{
    /**
     * 查询课程
     *
     * @param courseId 课程主键
     * @return 课程
     */
    public TbCourse selectTbCourseByCourseId(Long courseId);

    /**
     * 查询课程列表
     *
     * @param tbCourse 课程
     * @return 课程集合
     */
    public List<TbCourse> selectTbCourseList(TbCourse tbCourse);

    /**
     * 查询课程列表
     *
     * @return 课程集合
     */
    public List<TbCourse> getAllCourseList();

    /**
     * 新增课程
     *
     * @param tbCourse 课程
     * @return 结果
     */
    public int insertTbCourse(TbCourse tbCourse);

    /**
     * 修改课程
     *
     * @param tbCourse 课程
     * @return 结果
     */
    public int updateTbCourse(TbCourse tbCourse);

    /**
     * 修改课程
     *
     * @param tbCourse 课程
     * @return 结果
     */
    public int updateTbCourseById(TbCourse tbCourse);

    /**
     * 批量删除课程
     *
     * @param courseIds 需要删除的课程主键集合
     * @return 结果
     */
    public int deleteTbCourseByCourseIds(Long[] courseIds);

    /**
     * 删除课程信息
     *
     * @param courseId 课程主键
     * @return 结果
     */
    public int deleteTbCourseByCourseId(Long courseId);

    /**
     * 查询课程列表
     *
     * @param params 参数
     * @return ok
     */
    public List<Map<String, Object>> getCourseList(Map<String, Object> params);

    /**
     * 查询课程详情
     *
     * @param params 参数
     * @return ok
     */
    public Map<String, Object> getCourseInfo(Map<String, Object> params);

    /**
     * 立即参加
     *
     * @param courseDTO
     * @return ok
     */
    public AjaxResult joinNow(CourseDTO courseDTO);

    /**
     * 浏览量+1
     *
     * @param courseId 课程主键
     * @return 结果
     */
    public int addViewsNum(Long courseId);

    public PageInfo<Map<String, Object>> getCourseNameListForInput(Integer pageNum, Integer pageSize, String searchValue,Long memberUserId);
}
